* "xntg.f - 1.2 - 2009/06/15 NGS"
* --------------------------------------------------------------------
* For further technical information, questions, or comments:
*   NOAA, National Geodetic Survey, N/NGS6, Silver Spring, MD  U.S.A.
*   Attn   : Daniel R. Roman, Ph.D.
*   Phone  : 301-713-3202
*   Fax    : 301-713-4172
*   e-mail : dan.roman@noaa.gov
* --------------------------------------------------------------------

**********************************************************************
* Modification History -----
* 1999sep22 dsmith 1.0     for 1999 geoid models
* 
* 2003dec24 droman 1.1     for 2003 geoid models
*   Added endian functions, for platform dependant binary data byte-swapping.
*   This program can be used to convert an entire file
*   by "extracting" all of the binary grid into a binary sub-grid
*   (i.e., both input and output binary grids would be the same size).
* 
* 20090620 billw 1.2
*   added format statement when printing ASCII header
* 
* See sccs comments for future file history.
**********************************************************************

***********************************************************************
*                                                                     *
*                  DISCLAIMER                                         *
*                                                                     *
*   THIS PROGRAM AND SUPPORTING INFORMATION IS FURNISHED BY THE       *
* GOVERNMENT OF THE UNITED STATES OF AMERICA, AND IS ACCEPTED AND     *
* USED BY THE RECIPIENT WITH THE UNDERSTANDING THAT THE UNITED STATES *
* GOVERNMENT MAKES NO WARRANTIES, EXPRESS OR IMPLIED, CONCERNING THE  *
* ACCURACY, COMPLETENESS, RELIABILITY, OR SUITABILITY OF THIS         *
* PROGRAM, OF ITS CONSTITUENT PARTS, OR OF ANY SUPPORTING DATA.       *
*                                                                     *
*   THE GOVERNMENT OF THE UNITED STATES OF AMERICA SHALL BE UNDER NO  *
* LIABILITY WHATSOEVER RESULTING FROM ANY USE OF THIS PROGRAM.  THIS  *
* PROGRAM SHOULD NOT BE RELIED UPON AS THE SOLE BASIS FOR SOLVING A   *
* PROBLEM WHOSE INCORRECT SOLUTION COULD RESULT IN INJURY TO PERSON   *
* OR PROPERTY.                                                        *
*                                                                     *
*   THIS PROGRAM IS PROPERTY OF THE GOVERNMENT OF THE UNITED STATES   *
* OF AMERICA.  THEREFORE, THE RECIPIENT FURTHER AGREES NOT TO ASSERT  *
* PROPRIETARY RIGHTS THEREIN AND NOT TO REPRESENT THIS PROGRAM TO     *
* ANYONE AS BEING OTHER THAN A GOVERNMENT PROGRAM.                    *
*                                                                     *
***********************************************************************

      program xntg

**********************************************************************
* NGS program XNTG - "eXtract aNd Translate Geoid files"
* 
* Grid file data manipulation utility program
*  1) Extracts a sub-region of a geoid grid file (*asc or *.bin format)
*  2) Transforms geoid grid file format:
*     a) from ASCII  (*.asc) to binary (*.bin) format,  or
*     b) from binary (*.bin) to ASCII  (*.asc) format
* 
* NOTE: The grid file's binary (*.bin) format is unformatted, direct access.
*       The order of bytes depends on which platform the file was created. 
**********************************************************************
      implicit none
      CHARACTER*90  SCCSID
      CHARACTER*5   PGMVER
      CHARACTER*15  PGMDAT

c Functions
      integer   lnblnk

c Variables for the header of the input file
      real*8    glamn, glomn, dla, dlo
      integer*4 nla, nlo, ikind
      real*8    glamx, glomx

c Variables for the header of the output file
      real*8    glamno, glomno
      real*8    dlao,   dloo
      integer*4 nlao,   nloo, ikindo
      real*8    glamxo, glomxo

c Variables and functions for handling endian problem
      logical   reverse_bytes
      common    /endian/reverse_bytes
      real*8    frev8
      real*4    frev4
      integer*4 irev4

c Variables for statistics of input file
      real*4    mini, maxi
      integer*4 ilamini, ilomini, ilamaxi, ilomaxi, kounti
      real*8    xlamini, xlomini, xlamaxi, xlomaxi
      real*8    avei, stdi, rmsi, facti

c Variables for statistics of output file
      real*4    mino, maxo
      integer*4 ilamino, ilomino, ilamaxo, ilomaxo, kounto
      real*8    xlamino, xlomino, xlamaxo, xlomaxo
      real*8    aveo, stdo, rmso, facto

c Variables to get around the recl=4 issue for our real*8 header variables
      real*4    glamnx(2),  glomnx(2),  dlax(2),  dlox(2)
      real*4    glamnox(2), glomnox(2), dlaox(2), dloox(2)

      character*256  ifnam
      character*256  ofnam
      character*80   b80
      character*1    keyb
      character*80   nbound, sbound, wbound, ebound
      character*3    iext, oext
      integer        irec
      integer        itask
      integer        ifirst
      integer        ilast
      integer        ii
      integer        jj
      integer        iout
      integer*4      ilen
      integer*4      olen
      integer*4      lin, lout, ityp, otyp
      real*8         xla, dd1, dd2
      real*8         xlo

      real*4         ght(1081,1921)

c Needed for input header (8 byte variables, recl = 4)
      equivalence(glamnx(1), glamn)
      equivalence(glomnx(1), glomn)
      equivalence(dlax(1),   dla)
      equivalence(dlox(1),   dlo)

c Needed for output header (8 byte variables, recl = 4)
      equivalence(glamnox(1), glamno)
      equivalence(glomnox(1), glomno)
      equivalence(dlaox(1),   dlao)
      equivalence(dloox(1),   dloo)


ccccccccccccccccccccccccccccccccccccccccccccccccccc
c Initialize local variables
c Define the file numbers, and an 80 character blank
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      SCCSID="@(#)xntg.f   	1.2	2009/06/15 NGS"
      PGMVER="1.2"
      PGMDAT="2009/06/15"
      lin = 1
      lout = 10
      b80='                    '//
     *    '                    '//
     *    '                    '//
     *    '                    '

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c Write out the introductory/disclaimer screens
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      call intro(PGMVER,PGMDAT)

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Get input file name and type
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      write(6,101)
  101 format(/,1x,70('-'),/,
     *       ' What is the input file name? ',//,
     *       '  -> ',$)
      read(5,'(a)')ifnam
      ilen = lnblnk(ifnam)
      if(ifnam(ilen-2:ilen).eq.'asc')then
        ityp = 1
      elseif(ifnam(ilen-2:ilen).eq.'bin')then
        ityp = 2
      else
        write(6,201)
  201   format(/,' *** WARNING(201): I do not',
     *  ' recognize the extension of that file.')
  202   write(6,203)
  203   format(/,' Which format is that file? ',//,
     *  ' 1 = asc',/,
     *  ' 2 = bin',//,
     *  ' -> ',$)
        read(5,*)ityp
        if(ityp.lt.1 .or. ityp.gt.2)goto 202 
      endif

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Open input file and read header
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(ityp.eq.1)then
        open(lin,file=ifnam,status='old',form='formatted')
        read(lin,*)glamn,glomn,dla,dlo,nla,nlo,ikind
      else
        open(lin,file=ifnam,status='old',form='unformatted',
     *  access='direct',recl=4)
        read(lin,rec= 1)glamnx(1)
        read(lin,rec= 2)glamnx(2)
        read(lin,rec= 3)glomnx(1)
        read(lin,rec= 4)glomnx(2)
        read(lin,rec= 5)dlax(1)
        read(lin,rec= 6)dlax(2)
        read(lin,rec= 7)dlox(1)
        read(lin,rec= 8)dlox(2)
        read(lin,rec= 9)nla
        read(lin,rec=10)nlo
        read(lin,rec=11)ikind
      endif

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c check to see if it is necessary to byte-swap
c convert big endian values and to little endian
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(ikind.eq.1) then
        reverse_bytes=.FALSE.
      elseif(ikind.eq.irev4(1)) then
        reverse_bytes=.TRUE.
      else
        write(6,6115) ifnam
 6115   format('The file ',A, 'is in an unrecognized format.'/
     *         'xntg is terminating')
        stop
      endif

      if(reverse_bytes) then
        glamn= frev8 (glamn)
        glomn= frev8 (glomn)
        dla  = frev8 (dla)
        dlo  = frev8 (dlo)
        nla  = irev4 (nla)
        nlo  = irev4 (nlo)
        ikind= irev4 (ikind)
      endif

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Compute maximum lat/lon for input file
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      glamx = glamn + (nla-1)*dla
      glomx = glomn + (nlo-1)*dlo

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c Find out what we're gonna do with the file:
c      1) Extract?
c         A) If extracting, get the boundaries.
c      2a) Convert bin to asc?
c      2b) Convert asc to bin?
c      3) Just get statistics?
ccccccccccccccccccccccccccccccccccccccccccccccccccc

c - Deal with a *.asc file
      if(ityp.eq.1)then
        write(6,102)
  102   format(/,1x,70('-'),/,
     *       ' Which function to perform? ',//,
     *  ' 1 = Extract a sub-grid and convert "*.asc" to "*.bin"',/,
     *  ' 2 = Extract a sub-grid without converting format',/,
     *  ' 3 = Convert a "*.asc" file to "*.bin" ',/,
     *  ' 4 = Just give statistics on the file',//,
     *       '  -> ',$)

c - OR, Deal with a *.bin file
      else
        write(6,103)
  103   format(/,1x,70('-'),/,
     *       ' Which function to perform? ',//,
     *  ' 1 = Extract a sub-grid and convert "*.bin" to "*.asc"',/,
     *  ' 2 = Extract a sub-grid without converting format',/,
     *  ' 3 = Convert a "*.bin" file to "*.asc"',/,
     *  ' 4 = Just give statistics on the file',//,
     *       '  -> ',$)
      endif

      read(5,*)itask

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Determine the output file type
c - otyp = 0 => No output file
c - otyp = 1 => *.asc
c - otyp = 2 => *.bin
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(itask.eq.4)then
        otyp = 0
      elseif( (ityp.eq.1 .and. itask.eq.2) .or.
     *        (ityp.eq.2 .and. itask.eq.1) .or.
     *        (ityp.eq.2 .and. itask.eq.3) )then
        otyp = 1 
      else
        otyp = 2
      endif

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Get sub-grid boundaries if we're extracting
c - otherwise set "sub-grid boundaries" to the
c - boundaries of the input grid.
c - Don't allow sub-regions to exceed the
c - input boundaries
c - Use subroutine "bound" to convert the ascii
c - string of boundaries into a real*8 value
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(itask.eq.1 .or. itask.eq.2)then
        write(6,104) 
  104   format(/,1x,70('-'),/,
     *       ' Input the boundaries of the sub-grid in any of the',/,
     *       ' following two formats: ',/,
     *   '    (a) decimal/integer degrees',
     *   ' (one number)',/,
     *   '    (b) integer degrees,  decimal/integer minutes',
     *   ' (two numbers)',/,
     *   ' The latitude must be positive North.  The longitude',/,
     *   ' must be positive *EAST*, to accomodate U.S. territories',/,
     *   ' which are on either side of the international date line.',//)

c - North
  109   write(6,108)glamx
  108   format(' North Boundary (default: ',f10.6,') -> ',$)
        read(5,'(a80)')nbound
c - The following line fails with older FORTRAN compilers
c       if(nbound.eq.'')then
        if(nbound.eq.b80)then
          glamxo = glamx
        else
          call bound(nbound,glamxo)
        endif
        if(glamxo.gt.glamx)then
          write(6,*) ' North boundary too large...try again...'
          goto 109 
        endif

c - South
  110   write(6,105)glamn
  105   format(' South Boundary (default: ',f10.6,') -> ',$)
        read(5,'(a80)')sbound
c - The following line fails with older FORTRAN compilers
c       if(sbound.eq.'')then
        if(sbound.eq.b80)then
          glamno = glamn
        else
          call bound(sbound,glamno)
        endif
        if(glamno.lt.glamn)then
          write(6,*) ' South boundary too small...try again...'
          goto 110
        elseif(glamno.gt.glamxo)then
          write(6,*) ' South boundary too large...try again...'
          goto 110 
        endif
        
c - West
  111   write(6,106)glomn
  106   format(' West  Boundary (default: ',f10.6,') -> ',$)
        read(5,'(a80)')wbound
c - The following line fails with older FORTRAN compilers
c       if(wbound.eq.'')then
        if(wbound.eq.b80)then
          glomno = glomn
        else
          call bound(wbound,glomno)
        endif
        if(glomno.lt.glomn)then
          write(6,*) ' West boundary too small...try again...'
          goto 111
        endif

c - East
  112   write(6,107)glomx
  107   format(' East  Boundary (default: ',f10.6,') -> ',$)
        read(5,'(a80)')ebound
c - The following line fails with older FORTRAN compilers
c       if(ebound.eq.'')then
        if(ebound.eq.b80)then
          glomxo = glomx
        else
          call bound(ebound,glomxo)
        endif
        if(glomxo.gt.glomx)then
          write(6,*) ' East boundary too large...try again...'
          goto 112
        elseif(glomxo.lt.glomno)then
          write(6,*) ' East boundary too small...try again...'
          goto 112 
        endif

c - No extraction, just set output boundaries to input boundaries
      else
        glamno = glamn
        glamxo = glamx
        glomno = glomn
        glomxo = glomx
      endif

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Compute the header of the output file, and
c - some other relevant information
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      dlao = dla
      dloo = dlo

      nlao = nint((glamxo - glamno) / dlao) + 1
      nloo = nint((glomxo - glomno) / dloo) + 1

      ikindo = ikind

      ifirst = nint((glomno - glomn) / dloo) + 1 
      ilast  = nint((glomxo - glomn) / dloo) + 1

      if(ilast-ifirst+1 .ne. nloo)stop 88888  
 
ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Get output file name, if necessary, and open
c - that file.
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(otyp.ne.0)then
        write(6,113)
  113   format(/,1x,70('-'),/,
     *         ' What is the output file name? ',//,
     *         '  -> ',$)
        read(5,'(a)')ofnam
        olen = lnblnk(ofnam)

c - Output to be *.asc
        if(otyp.eq.1)then
          open(lout,file=ofnam,status='new',form='formatted')

c - Output to be *.bin
        else
          open(lout,file=ofnam,status='new',form='unformatted',
     *    access='direct',recl=4)
        endif
      endif
      
ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Read the input data
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(ityp.eq.1)then
        do 114 ii = 1,nla
          read(lin,9001)(ght(ii,jj),jj=1,nlo)
  114   continue
      else
        do 115 ii = 1,nla
          do 116 jj = 1,nlo 
            irec = 11 + (ii-1)*nlo + jj
            read(lin,rec=irec)ght(ii,jj)
c read big endian values and convert to little endian
            if(reverse_bytes) ght(ii,jj)=frev4(ght(ii,jj))
  116     continue
  115   continue
      endif

ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Compute input file statistics
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      avei=0.d0
      stdi=0.d0
      rmsi=0.d0
      kounti = nla*nlo
      maxi = -99999999.
      mini = +99999999.
      
      do 301 ii=1,nla
        xla = glamn + (ii-1)*dla
        do 302 jj=1,nlo
          xlo  = glomn + (jj-1)*dlo
          avei = avei  + ght(ii,jj)
          rmsi = rmsi  + ght(ii,jj)*ght(ii,jj)

          if(ght(ii,jj).lt.mini)then
            mini    = ght(ii,jj)
            xlamini = xla
            xlomini = xlo
            ilamini = ii
            ilomini = jj
          endif
          if(ght(ii,jj).gt.maxi)then
            maxi    = ght(ii,jj)
            xlamaxi = xla
            xlomaxi = xlo
            ilamaxi = ii
            ilomaxi = jj
          endif
  302   continue
  301 continue
      avei  = avei / kounti
      rmsi  = sqrt(rmsi / kounti)
      facti = dble(kounti) / dble(kounti - 1)
      stdi  = sqrt(facti*(rmsi**2 - avei**2))
        
ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Write out the new file, extracting, translating
c - and computing statistics on the fly
c - PC version
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(otyp.ne.0)then
        aveo   = 0.d0
        stdo   = 0.d0
        rmso   = 0.d0
        kounto = nlao*nloo
        maxo   = -99999999.
        mino   = +99999999.

c - *.asc output
        if(otyp.eq.1)then
          iout = 0
          write(lout,128) glamno,glomno,dlao,dloo,nlao,nloo,ikindo
  128 format(f20.12,f19.11,e23.13,e23.13,3x,i4,2x,i4,2x,i1)

c - Spin through all input rows, but iout counts the output rows
          do 120 ii=1,nla
            xla = glamn  + (ii-1)*dla
            dd1 = glamxo - xla
            dd2 = xla    - glamno
            if(dd1.gt.-1d-8 .and. dd2.gt.-1.d-8)then

c For whatever reason, this gets quirky sometimes :(
c           if(xla.ge.glamno .and. xla.le.glamxo)then
              iout = iout + 1
              write(lout,9001)(ght(ii,jj),jj=ifirst,ilast)

c - Compile the statistics of the *.asc output file
              do 141 jj=ifirst,ilast 
                xlo  = glomn + (jj-1)*dlo
                aveo = aveo  + ght(ii,jj)
                rmso = rmso  + ght(ii,jj)*ght(ii,jj)
                if(ght(ii,jj).lt.mino) then
                  mino    = ght(ii,jj)
                  xlamino = xla
                  xlomino = xlo
                  ilamino = iout
                  ilomino = jj-ifirst+1 
                endif
                if(ght(ii,jj).gt.maxo) then
                  maxo    = ght(ii,jj)
                  xlamaxo = xla
                  xlomaxo = xlo
                  ilamaxo = iout
                  ilomaxo = jj-ifirst+1
                endif
  141         continue

            endif
  120     continue
        
c *.bin output
        else 
          write(lout,rec= 1) glamnox(1)
          write(lout,rec= 2) glamnox(2)
          write(lout,rec= 3) glomnox(1)
          write(lout,rec= 4) glomnox(2)
          write(lout,rec= 5) dlaox(1)
          write(lout,rec= 6) dlaox(2)
          write(lout,rec= 7) dloox(1)
          write(lout,rec= 8) dloox(2)
          write(lout,rec= 9) nlao
          write(lout,rec=10) nloo
          write(lout,rec=11) ikind

          iout = 0
          irec = 11

c Spin through all input rows, but iout counts the output rows
          do 130 ii=1,nla
            xla = glamn  + (ii-1)*dla
            dd1 = glamxo - xla
            dd2 = xla    - glamno
            if(dd1.gt.-1d-8 .and. dd2.gt.-1.d-8) then

c For whatever reason, this gets quirky sometimes :(
c           if(xla.ge.glamno .and. xla.le.glamxo)then

              iout = iout + 1
              do 160 jj=ifirst,ilast
                irec = irec + 1
                write(lout,rec=irec)ght(ii,jj)
  160         continue

c - Compile some statistics
              do 132 jj=ifirst,ilast
                xlo  = glomn + (jj-1)*dlo
                aveo = aveo  + ght(ii,jj)
                rmso = rmso  + ght(ii,jj)*ght(ii,jj)

                if(ght(ii,jj).lt.mino) then
                  mino    = ght(ii,jj)
                  xlamino = xla
                  xlomino = xlo
                  ilamino = iout
                  ilomino = jj - ifirst + 1
                endif
                if(ght(ii,jj).gt.maxo) then
                  maxo    = ght(ii,jj)
                  xlamaxo = xla
                  xlomaxo = xlo
                  ilamaxo = iout
                  ilomaxo = jj - ifirst + 1
                endif

  132         continue

            endif
  130     continue
        endif  

c - Finish computing statistics for either ASCII or BIN output file
        aveo  = aveo / kounto
        rmso  = sqrt(rmso / kounto)
        facto = dble(kounto) / dble(kounto - 1)
        stdo  = sqrt(facto*(rmso*rmso - aveo*aveo))

      endif

 9001 format(8(1x,f9.4))


ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Write out final report on tasks completed
c - and statistics computed
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(ityp.eq.1) then
        iext = 'asc'
      else
        iext = 'bin'
      endif

      if(otyp.eq.0) then
        oext = '   ' 
      elseif(otyp.eq.1) then
        oext = 'asc'
      else
        oext = 'bin'
      endif

      write(6,143)
      write(6,144) ifnam(1:ilen)
      write(6,145) iext,nla,nlo,kounti,
     *   glamn, glamx, dla,
     *   glomn, glomx, dlo,
     *   mini, ilamini, ilomini, xlamini, xlomini,
     *   maxi, ilamaxi, ilomaxi, xlamaxi, xlomaxi,
     *   avei, stdi, rmsi

  143 format(/,1x,70('-'),/
     *' REPORT ON INPUT FILE:',/,
     *'     File Name                   : ',$)
  144 format(a)
  145 format(              
     *'     File type                   : ',a3,/,
     *'     Rows / Columns in file      : ',i8,1x,i8,/,
     *'     Number of points in file    : ',i12,/,
     *'     South Edge (Degrees North)  : ',f10.6,/,
     *'     North Edge (Degrees North)  : ',f10.6,/,
     *'     Latitude Spacing (Degrees)  : ',f10.6,/,
     *'     West  Edge (Degrees East)   : ',f10.6,/,
     *'     East  Edge (Degrees East)   : ',f10.6,/,
     *'     Longitude Spacing (Degrees) : ',f10.6,/,
     *'     Minimum geoid height        : ',f10.6,/,
     *'     -- Row/Col of minimum       : ',i8,2x,i8,/,
     *'     -- Lat/Lon of minimum       : ',f10.6,2x,f10.6,/,
     *'     Maximum geoid height        : ',f10.6,/,
     *'     -- Row/Col of maximum       : ',i8,2x,i8,/,
     *'     -- Lat/Lon of maximum       : ',f10.6,2x,f10.6,/,
     *'     Average geoid height        : ',f10.6,/,
     *'     Standard deviation          : ',f10.6,/,
     *'     Root Mean Square            : ',f10.6)

       
      if(otyp.ne.0) then
        write(6,150)
        read(5,'(a)')keyb

        write(6,146)
        write(6,147) ofnam(1:ilen)
        write(6,148) oext, nlao, nloo, kounto,
     *     glamno, glamxo, dlao,
     *     glomno, glomxo, dloo,
     *     mino, ilamino, ilomino, xlamino, xlomino,
     *     maxo, ilamaxo, ilomaxo, xlamaxo, xlomaxo,
     *     aveo, stdo, rmso
      endif

  150 format(/,'   <Hit RETURN to Continue>')
  146 format(/,1x,70('-'),/
     *  ' REPORT ON OUTPUT FILE:',/,
     *  '     File Name                   : ',$)
  147 format(a)
  148 format(              
     *  '     File type                   : ',a3,/,
     *  '     Rows / Columns in file      : ',i8,1x,i8,/,
     *  '     Number of points in file    : ',i12,/,
     *  '     South Edge (Degrees North)  : ',f10.6,/,
     *  '     North Edge (Degrees North)  : ',f10.6,/,
     *  '     Latitude Spacing (Degrees)  : ',f10.6,/,
     *  '     West  Edge (Degrees East)   : ',f10.6,/,
     *  '     East  Edge (Degrees East)   : ',f10.6,/,
     *  '     Longitude Spacing (Degrees) : ',f10.6,/,
     *  '     Minimum geoid height        : ',f10.6,/,
     *  '     -- Row/Col of minimum       : ',i8,2x,i8,/,
     *  '     -- Lat/Lon of minimum       : ',f10.6,2x,f10.6,/,
     *  '     Maximum geoid height        : ',f10.6,/,
     *  '     -- Row/Col of maximum       : ',i8,2x,i8,/,
     *  '     -- Lat/Lon of maximum       : ',f10.6,2x,f10.6,/,
     *  '     Average geoid height        : ',f10.6,/,
     *  '     Standard deviation          : ',f10.6,/,
     *  '     Root Mean Square            : ',f10.6)

      end

*** xntg.f
